@@ -1,8 +1,10 @@
 use 5.006;
 use strict;
 use warnings;
+
 package CPAN::Mini;
-our $VERSION = '1.100470_001';
+our $VERSION = '1.100590';
+
 # ABSTRACT: create a minimal mirror of CPAN
 
 ## no critic RequireCarping
@@ -11,14 +13,14 @@ our $VERSION = '1.100470_001';
 use Carp ();
 
 use File::Basename ();
-use File::Copy ();
-use File::HomeDir 0.57 (); # Win32 support
+use File::Copy     ();
+use File::HomeDir 0.57 ();  # Win32 support
 use File::Find ();
-use File::Path 2.04 (); # new API, bugfixes
+use File::Path 2.04 ();     # new API, bugfixes
 use File::Spec ();
 use File::Temp ();
 
-use URI 1 ();
+use URI 1            ();
 use LWP::UserAgent 5 ();
 
 use Compress::Zlib 1.20 ();
@@ -29,6 +31,10 @@ sub update_mirror {
   $self = $self->new(@_) unless ref $self;
 
   unless ($self->{offline}) {
+    $self->trace("Updating $self->{local}\n");
+    $self->trace("Mirroring from $self->{remote}\n");
+    $self->trace("=" x 63 . "\n");
+
     # mirrored tracks the already done, keyed by filename
     # 1 = local-checked, 2 = remote-mirrored
     $self->mirror_indices;
@@ -52,7 +58,7 @@ sub update_mirror {
   return $self->{changes_made};
 }
 
-sub _recent { $_[0]->{recent}{$_[1]} = 1 };
+sub _recent { $_[0]->{recent}{ $_[1] } = 1 }
 
 sub _write_out_recent {
   my ($self) = @_;
@@ -143,9 +149,12 @@ sub new {
 
   $self->{remote} = "$self->{remote}/" if substr($self->{remote}, -1) ne '/';
 
+  my $version = $class->VERSION;
+  $version = 'v?' unless defined $version;
+
   $self->{__lwp} = LWP::UserAgent->new(
-    agent      => "$class/" . $class->VERSION,
-    env_proxy  => 1,
+    agent     => "$class/$version",
+    env_proxy => 1,
     ($self->{no_conn_cache} ? () : (keep_alive => 5)),
   );
 
@@ -217,10 +226,8 @@ sub _install_indices {
 
     unlink $local_file;
 
-    File::Copy::copy(
-      File::Spec->catfile($self->{scratch}, split m{/}, $file),
-      $local_file,
-    );
+    File::Copy::copy(File::Spec->catfile($self->{scratch}, split m{/}, $file),
+      $local_file,);
 
     $self->{mirrored}{$local_file} = 1;
   }
@@ -233,10 +240,9 @@ sub mirror_file {
   $arg ||= {};
 
   # full URL
-  my $remote_uri
-    = eval { $path->isa('URI') }
-    ? $path
-    : URI->new_abs($path, $self->{remote})->as_string;
+  my $remote_uri = eval { $path->isa('URI') }
+                 ? $path
+                 : URI->new_abs($path, $self->{remote})->as_string;
 
   # native absolute file
   my $local_file = File::Spec->catfile(
@@ -270,16 +276,19 @@ sub mirror_file {
       $self->_recent($path);
       $self->trace(" ... updated\n");
       $self->{changes_made}++;
-    } elsif ($res->code != 304) { # not modified
-      warn(($self->{trace} ? "\n" : q{}) . "$remote_uri: " . $res->status_line .  "\n")
-        if $self->{errors};
+    } elsif ($res->code != 304) {  # not modified
+      warn(
+        ($self->{trace} ? "\n" : q{})
+        . "$remote_uri: "
+        . $res->status_line . "\n"
+      ) if $self->{errors};
       return;
     } else {
       $self->trace(" ... up to date\n");
     }
   }
 
-  if ($path =~ m{^authors/id}) {  # maybe fetch CHECKSUMS
+  if ($path =~ m{^authors/id}) {   # maybe fetch CHECKSUMS
     my $checksum_path
       = URI->new_abs("CHECKSUMS", $remote_uri)->rel($self->{remote})->as_string;
 
@@ -293,11 +302,13 @@ sub mirror_file {
 sub __do_filter {
   my ($self, $filter, $file) = @_;
   return unless $filter;
+
   if (ref($filter) eq 'ARRAY') {
     for (@$filter) {
       return 1 if $self->__do_filter($_, $file);
     }
   }
+
   if (ref($filter) eq 'CODE') {
     return $filter->($file);
   } else {
@@ -365,7 +376,7 @@ sub clean_file {
 
 sub trace {
   my ($self, $message) = @_;
-  print $message if $self->{trace};
+  print { $self->_trace_fh } $message;
 }
 
 
@@ -380,6 +391,11 @@ sub __homedir {
   return $homedir;
 }
 
+sub __homedir_configfile {
+  my ($class) = @_;
+  my $default = File::Spec->catfile($class->__homedir, '.minicpanrc');
+}
+
 sub __default_configfile {
   my ($self) = @_;
 
@@ -388,18 +404,16 @@ sub __default_configfile {
 }
 
 sub read_config {
-  my ($class) = @_;
+  my ($class, $options) = @_;
 
-  my $filename = File::Spec->catfile($class->__homedir, '.minicpanrc');
+  my $config_file = $class->config_file($options);
+  $class->trace("Using config from $config_file\n");
 
-  $filename = $class->__default_configfile unless -e $filename;
-  return unless -e $filename;
-
-  open my $config_file, '<', $filename
-    or die "couldn't open config file $filename: $!";
+  open my $config_fh, '<', $config_file
+    or die "couldn't open config file $config_file: $!";
 
   my %config;
-  while (<$config_file>) {
+  while (<$config_fh>) {
     chomp;
     next if /\A\s*\Z/sm;
     if (/\A(\w+):\s*(\S.*?)\s*\Z/sm) { $config{$1} = $2; }
@@ -417,6 +431,59 @@ sub read_config {
 }
 
 
+sub config_file {
+  my ($class, $options) = @_;
+
+  my $config_file = do {
+    if (defined eval { $options->{config_file} }) {
+      $options->{config_file};
+    } elsif (defined $ENV{CPAN_MINI_CONFIG}) {
+      $ENV{CPAN_MINI_CONFIG};
+    } elsif (defined $class->__homedir_configfile) {
+      $class->__homedir_configfile;
+    } elsif (defined $class->__default_configfile) {
+      $class->__default_configfile;
+    } else {
+      ();
+    }
+  };
+
+  return (
+    (defined $config_file && -e $config_file)
+    ? $config_file
+    : ()
+  );
+}
+
+sub __default_fh { *STDOUT{IO} }
+
+# stolen from IO::Interactive
+local (*DEV_NULL, *DEV_NULL2);
+my $dev_null;
+
+BEGIN {
+  pipe *DEV_NULL, *DEV_NULL2
+    or die "Internal error: can't create null filehandle";
+  $dev_null = \*DEV_NULL;
+}
+
+sub __quiet_fh { $dev_null }
+
+sub _trace_fh {
+  my ($either) = @_;
+
+  return do {
+    if (ref $either and defined $either->{trace} and !$either->{trace}) {
+      $either->__quiet_fh;
+    } elsif (eval { $either->can('_default_fh'); }) {
+      $either->__default_fh;
+    } else {
+      __default_fh();
+    }
+  };
+}
+
+
 1;
 
 __END__
@@ -428,7 +495,7 @@ CPAN::Mini - create a minimal mirror of CPAN
 
 =head1 VERSION
 
-version 1.100470_001
+version 1.100590
 
 =head1 SYNOPSIS
 
@@ -650,6 +717,18 @@ C<new> or C<update_mirror> methods.  It will look for a file called
 F<.minicpanrc> in the user's home directory as determined by
 L<File::HomeDir|File::HomeDir>.
 
+=head2 config_file
+
+  my %config = CPAN::Mini->config_file( { options } );
+
+This routine returns the config file name. It first looks at for the
+C<config_file> setting, then the C<CPAN_MINI_CONFIG> environment
+variable, then the default F<~/.minicpanrc>, and finally the
+F<CPAN/Mini/minicpan.conf>. It uses the first defined value it finds.
+If the filename it selects does not exist, it returns the empty list.
+
+OPTIONS is an optional hash reference of the C<CPAN::Mini> config hash. 
+
 =begin devel
 
 =method _filter_module
